@@ -81,9 +81,10 @@ subroutine getnp2 ( px, py, x, y, nr, lcell, lnext, xmin, ymin, &
 !    i1,i2,j1,j2 = cell indices of the layer whose intersection with 
 !    the range defined by imin,...,jmax is currently being searched.
 !
+  use iso_fortran_env
   implicit none
 
-  integer nr
+  integer(int64) nr
 
   real ( kind = 8 ) delx
   real ( kind = 8 ) dely
@@ -91,24 +92,24 @@ subroutine getnp2 ( px, py, x, y, nr, lcell, lnext, xmin, ymin, &
   real ( kind = 8 ) dx
   real ( kind = 8 ) dy
   logical first
-  integer i
-  integer i0
-  integer i1
-  integer i2
-  integer imax
-  integer imin
-  integer j
-  integer j0
-  integer j1
-  integer j2
-  integer jmax
-  integer jmin
-  integer l
-  integer lcell(nr,nr)
-  integer lmin
-  integer ln
-  integer lnext(*)
-  integer np
+  integer(int64) i
+  integer(int64) i0
+  integer(int64) i1
+  integer(int64) i2
+  integer(int64) imax
+  integer(int64) imin
+  integer(int64) j
+  integer(int64) j0
+  integer(int64) j1
+  integer(int64) j2
+  integer(int64) jmax
+  integer(int64) jmin
+  integer(int64) l
+  integer(int64) lcell(nr,nr)
+  integer(int64) lmin
+  integer(int64) ln
+  integer(int64) lnext(*)
+  integer(int64) np
   real ( kind = 8 ) px
   real ( kind = 8 ) py
   real ( kind = 8 ) r
@@ -141,11 +142,11 @@ subroutine getnp2 ( px, py, x, y, nr, lcell, lnext, xmin, ymin, &
   delx = xp - xmin
   dely = yp - ymin
 
-  i0 = int ( delx / dx ) + 1
+  i0 = int ( delx / dx, int64 ) + 1
   i0 = max ( i0, 1 )
   i0 = min ( i0, nr )
 
-  j0 = int ( dely / dy ) + 1
+  j0 = int ( dely / dy, int64 ) + 1
   j0 = max ( j0, 1 )
   j0 = min ( j0, nr )
 
@@ -206,16 +207,16 @@ subroutine getnp2 ( px, py, x, y, nr, lcell, lnext, xmin, ymin, &
             rsmin = rsq
             r = sqrt ( rsmin )
 
-            imin = int ( ( delx - r ) / dx ) + 1
+            imin = int ( ( delx - r ) / dx, int64 ) + 1
             imin = max ( imin, 1 )
 
-            imax = int ( ( delx + r ) / dx ) + 1
+            imax = int ( ( delx + r ) / dx, int64 ) + 1
             imax = min ( imax, nr )
 
-            jmin = int ( ( dely - r ) / dy ) + 1
+            jmin = int ( ( dely - r ) / dy, int64 ) + 1
             jmin = max ( jmin, 1 )
 
-            jmax = int ( ( dely + r ) / dy ) + 1
+            jmax = int ( ( dely + r ) / dy, int64 ) + 1
             jmax = min ( jmax, nr )
 
             first = .false.
@@ -328,6 +329,7 @@ subroutine qshep_givens ( a, b, c, s )
 !  Note that r has the sign of a, 0 < c, and s has
 !  sign(a)*sign(b).
 !
+  use iso_fortran_env
   implicit none
 
   real ( kind = 8 ) a
@@ -448,10 +450,11 @@ subroutine qs2grd ( px, py, n, x, y, f, nr, lcell, lnext, xmin, &
 !    2, if no errors were encountered but (PX,PY) is not within the 
 !       radius R(K) for any node K and thus Q = QX = QY = 0.
 !
+  use iso_fortran_env
   implicit none
 
-  integer n
-  integer nr
+  integer(int64) n
+  integer(int64) nr
 
   real ( kind = 8 ) a(5,n)
   real ( kind = 8 ) delx
@@ -460,17 +463,17 @@ subroutine qs2grd ( px, py, n, x, y, f, nr, lcell, lnext, xmin, &
   real ( kind = 8 ) dx
   real ( kind = 8 ) dy
   real ( kind = 8 ) f(n)
-  integer i
-  integer ier
-  integer imax
-  integer imin
-  integer j
-  integer jmax
-  integer jmin
-  integer k
-  integer kp
-  integer lcell(nr,nr)
-  integer lnext(n)
+  integer(int64) i
+  integer(int64) ier
+  integer(int64) imax
+  integer(int64) imin
+  integer(int64) j
+  integer(int64) jmax
+  integer(int64) jmin
+  integer(int64) k
+  integer(int64) kp
+  integer(int64) lcell(nr,nr)
+  integer(int64) lnext(n)
   real ( kind = 8 ) px
   real ( kind = 8 ) py
   real ( kind = 8 ) q
@@ -528,16 +531,16 @@ subroutine qs2grd ( px, py, n, x, y, f, nr, lcell, lnext, xmin, &
 !  sected by (or contained in) a circle of radius rmax
 !  centered at p.
 !
-  imin = int ( ( xp - xmin - rmax ) / dx ) + 1
+  imin = int ( ( xp - xmin - rmax ) / dx, int64 ) + 1
   imin = max ( imin, 1 )
 
-  imax = int ( ( xp - xmin + rmax ) / dx ) + 1
+  imax = int ( ( xp - xmin + rmax ) / dx, int64 ) + 1
   imax = min ( imax, nr )
 
-  jmin = int ( ( yp - ymin - rmax ) / dy ) + 1
+  jmin = int ( ( yp - ymin - rmax ) / dy, int64 ) + 1
   jmin = max ( jmin, 1 )
 
-  jmax = int ( ( yp - ymin + rmax ) / dy ) + 1
+  jmax = int ( ( yp - ymin + rmax ) / dy, int64 ) + 1
   jmax = min ( jmax, nr )
 !
 !  Test for no cells within the circle of radius RMAX.
@@ -824,10 +827,11 @@ subroutine qshep2 ( n, x, y, f, nq, nw, nr, lcell, lnext, xmin, &
 ! xk,yk =     coordinates of node k -- x(k), y(k)
 ! xmn,ymn =   local variables for xmin and ymin
 !
+  use iso_fortran_env
   implicit none
 
-  integer n
-  integer nr
+  integer(int64) n
+  integer(int64) nr
 
   real ( kind = 8 ) a(5,n)
   real ( kind = 8 ) av
@@ -842,25 +846,25 @@ subroutine qshep2 ( n, x, y, f, nq, nw, nr, lcell, lnext, xmin, &
   real ( kind = 8 ) dy
   real ( kind = 8 ) f(n)
   real ( kind = 8 ) fk
-  integer i
-  integer ier
-  integer ierr
-  integer irow
-  integer j
-  integer jp1
-  integer k
-  integer lcell(nr,nr)
-  integer lmax
-  integer lnext(n)
-  integer lnp
-  integer neq
-  integer nn
-  integer nnr
-  integer np
-  integer npts(40)
-  integer nq
-  integer nqwmax
-  integer nw
+  integer(int64) i
+  integer(int64) ier
+  integer(int64) ierr
+  integer(int64) irow
+  integer(int64) j
+  integer(int64) jp1
+  integer(int64) k
+  integer(int64) lcell(nr,nr)
+  integer(int64) lmax
+  integer(int64) lnext(n)
+  integer(int64) lnp
+  integer(int64) neq
+  integer(int64) nn
+  integer(int64) nnr
+  integer(int64) np
+  integer(int64) npts(40)
+  integer(int64) nq
+  integer(int64) nqwmax
+  integer(int64) nw
   real ( kind = 8 ) rmax
   real ( kind = 8 ) rq
   real ( kind = 8 ) rs
@@ -1245,10 +1249,11 @@ function qs2val ( px, py, n, x, y, f, nr, lcell, lnext, xmin, &
 !    Output, real ( kind = 8 ) QS2VAL, the interpolated function value
 !    at (PX,PY).
 !
+  use iso_fortran_env
   implicit none
 
-  integer n
-  integer nr
+  integer(int64) n
+  integer(int64) nr
 
   real ( kind = 8 ) a(5,n)
   real ( kind = 8 ) delx
@@ -1256,17 +1261,17 @@ function qs2val ( px, py, n, x, y, f, nr, lcell, lnext, xmin, &
   real ( kind = 8 ) dx
   real ( kind = 8 ) dy
   real ( kind = 8 ) f(n)
-  integer i
-  integer imax
-  integer imin
-  integer j
-  integer jmax
-  integer jmin
+  integer(int64) i
+  integer(int64) imax
+  integer(int64) imin
+  integer(int64) j
+  integer(int64) jmax
+  integer(int64) jmin
   real ( kind = 8 ) ds
-  integer k
-  integer kp
-  integer lcell(nr,nr)
-  integer lnext(n)
+  integer(int64) k
+  integer(int64) kp
+  integer(int64) lcell(nr,nr)
+  integer(int64) lnext(n)
   real ( kind = 8 ) px
   real ( kind = 8 ) py
   real ( kind = 8 ) qs2val
@@ -1307,16 +1312,16 @@ function qs2val ( px, py, n, x, y, f, nr, lcell, lnext, xmin, &
 !  by (or contained in) a circle of radius rmax
 !  centered at P.
 !
-  imin = int ( ( xp - xmin - rmax ) / dx ) + 1
+  imin = int ( ( xp - xmin - rmax ) / dx, int64 ) + 1
   imin = max ( imin, 1 )
 
-  imax = int ( ( xp - xmin + rmax ) / dx ) + 1
+  imax = int ( ( xp - xmin + rmax ) / dx, int64 ) + 1
   imax = min ( imax, nr )
 
-  jmin = int ( ( yp - ymin - rmax ) / dy ) + 1
+  jmin = int ( ( yp - ymin - rmax ) / dy, int64 ) + 1
   jmin = max ( jmin, 1 )
 
-  jmax = int ( ( yp - ymin + rmax ) / dy ) + 1
+  jmax = int ( ( yp - ymin + rmax ) / dy, int64 ) + 1
   jmax = min ( jmax, nr )
 !
 !  Test for no cells within the circle of radius RMAX.
@@ -1420,12 +1425,13 @@ subroutine rotate ( n, c, s, x, y )
 !
 !    Input/output, real ( kind = 8 ) X(N), Y(N), the rotated vectors. 
 !
+  use iso_fortran_env
   implicit none
 
-  integer n
+  integer(int64) n
 
   real ( kind = 8 ) c
-  integer i
+  integer(int64) i
   real ( kind = 8 ) s
   real ( kind = 8 ) x(n)
   real ( kind = 8 ) xi
@@ -1493,6 +1499,7 @@ subroutine setup2 ( xk, yk, fk, xi, yi, fi, s1, s2, r, row )
 !
 !    Output, real ( kind = 8 ) ROW(6), a row of the augmented regression matrix.
 !
+  use iso_fortran_env
   implicit none
 
   real ( kind = 8 ) d
@@ -1615,20 +1622,21 @@ subroutine store2 ( n, x, y, nr, lcell, lnext, xmin, ymin, dx, dy, ier )
 !    1, if N < 2 or NR < 1.
 !    2, if DX = 0 or DY = 0.
 !
+  use iso_fortran_env
   implicit none
 
-  integer n
-  integer nr
+  integer(int64) n
+  integer(int64) nr
 
   real ( kind = 8 ) dx
   real ( kind = 8 ) dy
-  integer i
-  integer ier
-  integer j
-  integer k
-  integer l
-  integer lcell(nr,nr)
-  integer lnext(n)
+  integer(int64) i
+  integer(int64) ier
+  integer(int64) j
+  integer(int64) k
+  integer(int64) l
+  integer(int64) lcell(nr,nr)
+  integer(int64) lnext(n)
   real ( kind = 8 ) x(n)
   real ( kind = 8 ) xmax
   real ( kind = 8 ) xmin
@@ -1675,10 +1683,10 @@ subroutine store2 ( n, x, y, nr, lcell, lnext, xmin, ymin, dx, dy, ier )
 !
   do k = n, 1, -1
 
-    i = int ( ( x(k) - xmin ) / dx ) + 1
+    i = int ( ( x(k) - xmin ) / dx, int64 ) + 1
     i = min ( i, nr )
 
-    j = int ( ( y(k) - ymin ) / dy ) + 1
+    j = int ( ( y(k) - ymin ) / dy, int64 ) + 1
     j = min ( j, nr )
 
     l = lcell(i,j)
