@@ -93,12 +93,10 @@ typedef unsigned long uint64_t;
 
 #include "io_binary.h"
 
-typedef char byte;
-
 static const int size_of[6]      = {4, 8, 8, 16, 4, 8};
 
 void FC_FUNC_(read_csv,READ_CSV)
-     (unsigned long * np, byte * f, int * output_type, int * ierr, STR_F_TYPE fname STR_ARG1)
+     (unsigned long * np, void * f, int * output_type, int * ierr, STR_F_TYPE fname STR_ARG1)
 {
   char * filename;
   int i;
