@@ -27,28 +27,28 @@
 #endif
 
 {
-  const int n = opn[0];
-  const int nri = opnri[0];
+  const ptrdiff_t n = opn[0];
+  const ptrdiff_t nri = opnri[0];
 #if DEPTH >= 32
-  const int unroll32 = max1(32*VEC_SIZE >> ldf);
+  const ptrdiff_t unroll32 = max1(32*VEC_SIZE >> ldf);
 #endif
 #if DEPTH >= 16
-  const int unroll16 = max1(16*VEC_SIZE >> ldf);
+  const ptrdiff_t unroll16 = max1(16*VEC_SIZE >> ldf);
 #endif
 #if DEPTH >= 8
-  const int unroll8  = max1(8*VEC_SIZE >> ldf);
+  const ptrdiff_t unroll8  = max1(8*VEC_SIZE >> ldf);
 #endif
 #if DEPTH >= 4
-  const int unroll4  = max1(4*VEC_SIZE >> ldf);
+  const ptrdiff_t unroll4  = max1(4*VEC_SIZE >> ldf);
 #endif
 #if DEPTH >= 2
-  const int unroll2  = max1(2*VEC_SIZE >> ldf);
+  const ptrdiff_t unroll2  = max1(2*VEC_SIZE >> ldf);
 #endif
 #if DEPTH >= 1
-  const int unroll1  = max1(1*VEC_SIZE >> ldf);
+  const ptrdiff_t unroll1  = max1(1*VEC_SIZE >> ldf);
 #endif
 
-  int l, i, j;
+  ptrdiff_t l, i, j;
   const int * restrict index;
    
   for (l = 0; l < nri ; l++) {
@@ -57,7 +57,7 @@
 
 #if DEPTH >= 32
     for (; i < (rimap_inv_max[l] - unroll32 + 1) ; i += unroll32){
-      int k;
+      ptrdiff_t k;
       for(k = 0; k < (1<<ldf); k += 32*VEC_SIZE){
 	register VEC_TYPE a0, a1, a2, a3;
 	register VEC_TYPE a4, a5, a6, a7;
@@ -83,7 +83,7 @@
 #else	  
 	  register VEC_TYPE wj = VEC_SCAL(w[j]);
 #endif
-	  int indexj = (index[j] + i)<<ldf;
+	  ptrdiff_t indexj = (index[j] + i)<<ldf;
 	  a0 = VEC_FMA(wj, LOAD(fi + indexj              + k), a0);
 	  a1 = VEC_FMA(wj, LOAD(fi + indexj + 1*VEC_SIZE + k), a1);
 	  a2 = VEC_FMA(wj, LOAD(fi + indexj + 2*VEC_SIZE + k), a2);
@@ -156,7 +156,7 @@
 
 #if DEPTH >= 16
     for (; i < (rimap_inv_max[l] - unroll16 + 1) ; i += unroll16){
-      int k;
+      ptrdiff_t k;
       for(k = 0; k < (1<<ldf); k += 16*VEC_SIZE){
 	register VEC_TYPE a0, a1, a2, a3;
 	register VEC_TYPE a4, a5, a6, a7;
@@ -174,7 +174,7 @@
 #else
 	  register VEC_TYPE wj = VEC_SCAL(w[j]);
 #endif
-	  int indexj = (index[j] + i)<<ldf;
+	  ptrdiff_t indexj = (index[j] + i)<<ldf;
 	  a0 = VEC_FMA(wj, LOAD(fi + indexj              + k), a0);
 	  a1 = VEC_FMA(wj, LOAD(fi + indexj + 1*VEC_SIZE + k), a1);
 	  a2 = VEC_FMA(wj, LOAD(fi + indexj + 2*VEC_SIZE + k), a2);
@@ -214,7 +214,7 @@
 
 #if DEPTH >= 8
     for (; i < (rimap_inv_max[l] - unroll8 + 1) ; i += unroll8){
-      int k;
+      ptrdiff_t k;
       for(k = 0; k < (1<<ldf); k += 8*VEC_SIZE){
 	register VEC_TYPE a0, a1, a2, a3;
 	register VEC_TYPE a4, a5, a6, a7;
@@ -228,7 +228,7 @@
 #else
 	  register VEC_TYPE wj = VEC_SCAL(w[j]);
 #endif
-	  int indexj = (index[j] + i)<<ldf;
+	  ptrdiff_t indexj = (index[j] + i)<<ldf;
 	  a0 = VEC_FMA(wj, LOAD(fi + indexj              + k), a0);
 	  a1 = VEC_FMA(wj, LOAD(fi + indexj + 1*VEC_SIZE + k), a1);
 	  a2 = VEC_FMA(wj, LOAD(fi + indexj + 2*VEC_SIZE + k), a2);
@@ -252,7 +252,7 @@
 
 #if DEPTH >= 4
     for (; i < (rimap_inv_max[l] - unroll4 + 1) ; i += unroll4){
-      int k;
+      ptrdiff_t k;
       for(k = 0; k < (1<<ldf); k += 4*VEC_SIZE){
 	register VEC_TYPE a0, a1, a2, a3;
 
@@ -264,7 +264,7 @@
 #else
 	  register VEC_TYPE wj = VEC_SCAL(w[j]);
 #endif
-	  int indexj = (index[j] + i)<<ldf;
+	  ptrdiff_t indexj = (index[j] + i)<<ldf;
 	  a0 = VEC_FMA(wj, LOAD(fi + indexj              + k), a0);
 	  a1 = VEC_FMA(wj, LOAD(fi + indexj + 1*VEC_SIZE + k), a1);
 	  a2 = VEC_FMA(wj, LOAD(fi + indexj + 2*VEC_SIZE + k), a2);
@@ -280,7 +280,7 @@
 
 #if DEPTH >= 2
     for (; i < (rimap_inv_max[l] - unroll2 + 1) ; i += unroll2){
-      int k;
+      ptrdiff_t k;
       for(k = 0; k < (1<<ldf); k += 2*VEC_SIZE){
 	register VEC_TYPE a0, a1;
 
@@ -292,7 +292,7 @@
 #else
           register VEC_TYPE wj = VEC_SCAL(w[j]);
 #endif
-	  int indexj = (index[j] + i)<<ldf;
+	  ptrdiff_t indexj = (index[j] + i)<<ldf;
 	  a0 = VEC_FMA(wj, LOAD(fi + indexj              + k), a0);
 	  a1 = VEC_FMA(wj, LOAD(fi + indexj + 1*VEC_SIZE + k), a1);
 	}
@@ -304,7 +304,7 @@
 
 #if DEPTH >= 1
     for (; i < (rimap_inv_max[l] - unroll1 + 1) ; i += unroll1){
-      int k;
+      ptrdiff_t k;
       for(k = 0; k < (1<<ldf); k += VEC_SIZE){
 	register VEC_TYPE a0 = VEC_ZERO;
 	for(j = 0; j < n; j++) {
@@ -313,7 +313,7 @@
 #else
 	  register VEC_TYPE wj = VEC_SCAL(w[j]);
 #endif
-	  int indexj = (index[j] + i)<<ldf;
+	  ptrdiff_t indexj = (index[j] + i)<<ldf;
 	  a0 = VEC_FMA(wj, LOAD(fi + indexj + k), a0);
 	}
 	STORE(fo + (i<<ldf) + k, a0);
@@ -323,7 +323,7 @@
 
 #if VEC_SIZE > 1
     for (; i < rimap_inv_max[l]; i++){
-      int k;
+      ptrdiff_t k;
       for(k = 0; k < (1<<ldf); k++){
 	double a = 0.0;
 	for(j = 0; j < n; j++) a += w[j]*fi[((index[j] + i)<<ldf) + k];
