@@ -90,6 +90,24 @@
 
 #  define SAFE_ALLOCATE_TYPE_ARRAY(type, x, bounds) allocate(type::x bounds)
 
+#  define SAFE_ALLOCATE_SOURCE(x, y) \
+  allocate(x, source=y); CARDINAL \
+  CARDINAL
+
+#  define SAFE_ALLOCATE_SOURCE_A(x, y) \
+  if(allocated(y)) then;   CARDINAL \
+    allocate(x, source=y); CARDINAL \
+  end if; \
+  CARDINAL
+
+#  define SAFE_ALLOCATE_SOURCE_P(x, y) \
+  if(associated(y)) then;  CARDINAL \
+    allocate(x, source=y); CARDINAL \
+  else;                    CARDINAL \
+    nullify(x);            CARDINAL \
+  end if; \
+  CARDINAL
+
 #  define SAFE_DEALLOCATE_P(x) \
   if(associated(x)) then; CARDINAL \
     deallocate(x);        CARDINAL \
@@ -133,14 +151,38 @@
     call alloc_error(global_sizeof, ACARDINAL __FILE__, ACARDINAL __LINE__); \
   CARDINAL
 
+#  define SAFE_ALLOCATE_SOURCE_P(x, y) \
+  if(associated(y)) then;     CARDINAL \
+    allocate( ACARDINAL x, ACARDINAL source=y, ACARDINAL stat=global_alloc_err); CARDINAL \
+    SAFE_ALLOCATE_PROFILE(x); CARDINAL \
+  else;                       CARDINAL \
+    nullify(x);               CARDINAL \
+  end if; \
+  CARDINAL
+
+#  define SAFE_ALLOCATE_SOURCE_A(x, y) \
+  if(allocated(y)) then; CARDINAL \
+    allocate( ACARDINAL x, ACARDINAL source=y, ACARDINAL stat=global_alloc_err); CARDINAL \
+    SAFE_ALLOCATE_PROFILE(x); CARDINAL \
+  end if; \
+  CARDINAL
+
+#  define SAFE_ALLOCATE_SOURCE(x, y) \
+  allocate( ACARDINAL x, ACARDINAL source=y, ACARDINAL stat=global_alloc_err); CARDINAL \
+  SAFE_ALLOCATE_PROFILE(x); CARDINAL \
+  CARDINAL
+
 #  define MY_DEALLOCATE(x) \
-  global_sizeof = SIZEOF(x); \
+  global_sizeof = SIZEOF(x) ; \
   CARDINAL \
-  deallocate(x, stat=global_alloc_err); CARDINAL \
+  deallocate(x, stat=global_alloc_err, errmsg=global_alloc_errmsg); CARDINAL \
   if(not_in_openmp() .and. iand(prof_vars%mode, PROFILING_MEMORY).ne.0) ACARDINAL \
     call profiling_memory_deallocate(TOSTRING(x), ACARDINAL __FILE__, ACARDINAL __LINE__, ACARDINAL global_sizeof); CARDINAL \
-  if(global_alloc_err.ne.0) ACARDINAL \
-    call dealloc_error(global_sizeof, ACARDINAL __FILE__, ACARDINAL __LINE__)
+  if(global_alloc_err.ne.0) then; CARDINAL \
+    write(stderr,'(a)') global_alloc_errmsg; CARDINAL \
+    call dealloc_error(global_sizeof, ACARDINAL __FILE__, ACARDINAL __LINE__); CARDINAL \
+  end if; \
+  CARDINAL
 
 #  define SAFE_DEALLOCATE_P(x) \
   if(associated(x)) then; CARDINAL \
@@ -158,10 +200,6 @@
 #endif
 
 
-! This was used in the past and should not be used any more.
-#define ALLOCATE(a,b) _DEPRECATED_PLEASE_USE_SAFE_ALLOCATE_
-
-
 ! The following macros facilitate the use of real or complex variables,
 ! and the possibility of compiling the code in single or double precision.
 #define REAL_DOUBLE real(8)
@@ -177,32 +215,21 @@
 #define ZPREC(x)   	 z ## x
 #define CNST(x)   	 x ## _8
 #ifdef HAVE_LIBXC5
-#define XC_F90(x)      xc_f90_ ## x
 #define XC_SIZE_T c_size_t
 #else
-#define XC_F90(x)      xc_f03_ ## x
 #define XC_SIZE_T 4
 #endif
 
 #define   TOFLOAT(x) real(x, REAL_PRECISION)
 #define   TOCMPLX(x, y) cmplx(x, y, REAL_PRECISION)
 
-! The MPI1 and MPI2 standards are different regarding the MPI_IN_PLACE constant. In
-! the code, just use the MPI_IN_PLACE_OR defined here.
-#ifdef HAVE_MPI
-#ifdef HAVE_MPI2
-#define MPI_IN_PLACE_OR(x) MPI_IN_PLACE
-#else
-#define MPI_IN_PLACE_OR(x) x
-#endif
-#endif
+#define SAFE_TOL(x, tol) sign(max(abs(x),tol),x)
 
 
 ! the TOSTRING macro converts a macro into a string
 ! do not use the STRINGIFY macro
 #define STRINGIFY(x) #x
 #define TOSTRING(x)  STRINGIFY(x)
-#define INCR(x, y) x = (x) + (y)
 
 
 ! Whenever a procedure is not called too many times, one should start it
@@ -220,9 +247,6 @@
   endif; endif; \
   CARDINAL
 
-! the leading dimension of the array
-#define LD(a) ubound(a,dim=1)
-
 !! Local Variables:
 !! mode: f90
 !! coding: utf-8
