@@ -25,7 +25,9 @@
 #include <sstream>
 #include <iostream>
 
+#include "element.hpp"
 #include "base.hpp"
+
 #include <rapidxml.hpp>
 
 namespace pseudopotential {
@@ -38,10 +40,15 @@ namespace pseudopotential {
       file_(filename.c_str()),
       buffer_((std::istreambuf_iterator<char>(file_)), std::istreambuf_iterator<char>()){
 
+      filename_ = filename;
+      
       buffer_.push_back('\0');
       doc_.parse<0>(&buffer_[0]);
 
       root_node_ = doc_.first_node("fpmd:species");
+      //for the old version of the standard
+      if(!root_node_) root_node_ = doc_.first_node("qbox:species");
+      if(!root_node_) throw status::FORMAT_NOT_SUPPORTED;
 
       pseudo_node_ = root_node_->first_node("ultrasoft_pseudopotential");
       if(pseudo_node_) type_ = pseudopotential::type::ULTRASOFT;
@@ -84,7 +91,7 @@ namespace pseudopotential {
     }
     
     std::string symbol() const {
-      return root_node_->first_node("symbol")->value();
+      return element::trim(root_node_->first_node("symbol")->value());
     }
 
     int atomic_number() const {
